UP = '\x00H'
DOWN = '\x00P'
LEFT = '\x00K'
RIGHT = '\x00M'

CTRL_C = '\x03'
CTRL_RIGHT = '\x00t'
CTRL_LEFT = '\x00s'
CTRL_BACKSPACE = '\x7f'
CTRL_DELETE = '\x00\xc2\x93'

ENTER = '\r'
BACKSPACE = '\x08'
DELETE = '\x00S'

SPACE = ' '